/*                             Scramdisk for Linux
 *                             ===================
 *
 *  $Id: getiv.c,v 1.2 2005/08/11 20:49:29 huj Exp $
 *  Author: Hans-Ulrich Juettner
 *  Copyright (c) 2005 - Hans-Ulrich Juettner
 *
 *  Scramdisk for Linux is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  Scramdisk for Linux is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Scramdisk for Linux -- see the file COPYING. If not,
 *  write to the Free Software Foundation, Inc., 59 Temple Place,
 *  Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Scramdisk for Linux is a Windows container-compatible on-the-fly
 *  encryption system.  For details, see the Scramdisk World-Wide-Web page,
 *  `http://www.samsimpson.com/cryptography/scramdisk/', or send a mail to
 *  the Scramdisk for Linux developer <hans-ulrich.juettner@t-online.de>.
 *
 *  $Log: getiv.c,v $
 *  Revision 1.2  2005/08/11 20:49:29  huj
 *  Changes for installation on Debian
 *
 *  Revision 1.1  2005/05/22 20:10:22  huj
 *  First public version 0.8.2
 *
 *  Revision 1.1  2003/11/23 15:59:32  huj
 *  First version
 *
 */

#include "getiv.h"

typedef unsigned long long uint64;

int add_carry(uint32* a, uint32 b, int c);

int add_carry(uint32* a, uint32 b, int c)
{
  uint64 x = (uint64) *a;
  x += (uint64) b;
  if (c)
    ++x;
  *a = (uint32) x;
  return (int) (x >> 32);
}

void getiv64(uint32* l, uint32* r, uint32* cbccl, uint32* cbccr,
             int sectnum, octet* pwhite)
{
  // Shaun's code uses a table containing 32x128-bit values (A)
  // and 32x128-bit values (B) arranged as A,B,A,B,A,B,A,B,...
  // pwhite is a pointer to the 1K whitening table
  int n;
  int odd;
  int carry;
  uint32* ptable = (uint32*) pwhite;
  *l = 0U;
  *r = 0U;
  *cbccl = 0U;
  *cbccr = 0U;
  for (n = 0; n < 32; ++n)
  {
    odd = sectnum & 1;
    if (odd)
      ptable += 4;
    carry = add_carry(l, *(ptable++), 0);
    carry = add_carry(r, *(ptable++), carry);
    carry = add_carry(cbccl, *(ptable++), carry);
    carry = add_carry(cbccr, *(ptable++), carry);
    sectnum >>= 1;
    if (!odd)
      ptable += 4;
  }
}
